/*
 * File tinedefs.h
 *
 * Authors Philip Duval (DESY), Steve Herb (DESY), Marina Nikolova (EMBL), 
 *    Karol Bartkiewicz (DESY), Marcus Walla (DESY), Jan Szczesny (DESY),
 *    Arthur Aghababyan (DESY), Stefan Weisse (DESY), Tobias Tempel (DESY)
 *
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023
 *
 * The TINE .c codeset made available by open source license ultimately
 * stems from the original ISOLDE control system ((c)CERN, released under
 * a permissive CERN licence akin to the later well known MIT licence)
 * in use at CERN in 1992.  Although none of the current software is in
 * anyway a redistribution of software from the ISOLDE system of that
 * era (apart from various error/status code definitions), it does in
 * fact inherit several conceptual ideas.  We therefore owe a substantial
 * debt to CERN for its inception, and would be remiss if we did not express
 * our gratitude in the TINE open source headers.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _TINEDEFS_H_
#define _TINEDEFS_H_

#include "project.def"
#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#if defined(MACOS) || defined(UNIX)
# if !defined(USE_IPV6) && !defined(USE_IPV4)
#   define USE_IPV6
# endif
#elif defined(VXWORKS)
# ifndef MULTITHREADED
#   define MULTITHREADED
# endif
# ifndef HAVE_STRTOLL
#   define NEED_STRTOLL
# endif
# ifdef NEED_STRTOLL
#   ifndef SINT64
#     define SINT64 long long
#   endif
#   ifndef UINT64
#     define UINT64 unsigned long long
#   endif
    SINT64 strtoll(const char *nptr, char **endptr, int base);
# endif
#endif

#ifndef TINE_EXPORT
# define TINE_EXPORT
#endif

#ifndef CNTXT_PREFIX
#define CNTXT_PREFIX "\\\\"
#endif

#ifndef DELIMITER
#define DELIMITER "/\\"
#endif

/** 
 * \brief Direction: Local to Local flag (no byte swapping)
 */
#ifndef LTOL
# define LTOL   -1
#endif
/** 
 * \brief Direction: Remote to Local flag
 */
#ifndef RTOL
# define RTOL    0
#endif
/**
 * \brief  Direction: Local to Remote flag
 */  
#ifndef LTOR
# define LTOR    1  
#endif


#ifndef MAX
/** 
 * \brief Returns the greater of two variables.
 */
#define MAX(a,b)  ((a) > (b) ? a : b)
#endif

#ifndef MIN
/**
 * \brief Returns the lesser of two variables 
 */
#define MIN(a,b)  ((a) < (b) ? a : b) 
#endif

#ifndef BYTE
# define BYTE unsigned char
#endif
#ifndef UINT8
# define UINT8 unsigned char
#endif
#ifndef SINT16
# define SINT16 short
#endif
#ifndef UINT16
# define UINT16 unsigned short
#endif
#ifndef UINT32
# define UINT32 unsigned int
#endif
#ifndef SINT32
# define SINT32 int
#endif
#if defined(UNIX) || defined(MACOS)
# if !defined(SINT64) && !defined(NO_SINT64)
#   ifdef OS_64BIT
#     define SINT64 long
#     define UINT64 unsigned long
#   else
#     define SINT64 long long
#     define UINT64 unsigned long long
#   endif
# endif
#elif defined(WIN32) && !defined(NO_SINT64) 
# ifndef SINT64
#   define SINT64 __int64
#   define UINT64 unsigned __int64
# endif
#endif
#ifndef WIN32
typedef void* HWND;
# ifndef WORD
#   define WORD UINT16
# endif
# ifndef DWORD
#   define DWORD UINT32
# endif
#endif

#ifndef TRUE
/**
 * \brief Definition of logical True 
 */
#define TRUE -1
#endif

#ifndef FALSE
/**
 * \brief Definition of logical False 
 */
#define FALSE 0
#endif

short getArrayType(char *fmt);
short getArrayRowLength(char *fmt);
short ftoi(char *fmt);
#define getFormatType(f) ftoi(f)
char *itof(int fmt);
short mtoi(char *mod);
char *itom(short accessMode);
short getElevatedMcaFormat(short fmt);
short getPrimaryFormat(short fmt);
TINE_EXPORT int GetNumberSegments(int fmt);
#define getNumberSegments(a1) GetNumberSegments(a1)
TINE_EXPORT int GetFormatSize(int fmt);
#define getFormatSize(a1) GetFormatSize(a1)
TINE_EXPORT int GetFormatHeaderSize(int fmt);
#define getFormatHeaderSize(a1) GetFormatHeaderSize(a1)
TINE_EXPORT int GetFormatDataType(int fmt);
#define getFormatDataType(a1) GetFormatDataType(a1)
TINE_EXPORT BYTE *GetFormatBodyPtr(void *dst, int fmt);
#define getFormatBodyPtr(a1,a2) GetFormatBodyPtr(a1,a2)
TINE_EXPORT BYTE **GetFormatBodyPtrAddr(void *dst, int fmt);
#define getFormatBodyPtrAddr(a1,a2) GetFormatBodyPtrAddr(a1,a2)
TINE_EXPORT UINT32 GetFormatBodyBufferSize(void *dst, int fmt);
#define getFormatBodyBufferSize(a1,a2) GetFormatBodyBufferSize(a1,a2)
TINE_EXPORT BYTE *GetFormatBodyBufferSizeAddr(void *dst, int fmt);
#define getFormatBodyBufferSizeAddr(a1,a2) GetFormatBodyBufferSizeAddr(a1,a2)
TINE_EXPORT int GetFormatTypeSize(int fmt);
#define getFormatTypeSize(a1) GetFormatTypeSize(a1)
TINE_EXPORT int GetFormatStandardCapacity(int fmt);
#define getFormatStandardCapacity(a1) GetFormatStandardCapacity(a1)
TINE_EXPORT char *GetModeAsString(int transferMode);
TINE_EXPORT short GetTransferMode(char *mod);
TINE_EXPORT short GetAccessMode(char *mod);
TINE_EXPORT char *GetAccessAsString(int accessMode);
TINE_EXPORT char *GetFormatAsString(int fmt);
TINE_EXPORT short GetFormatFromString(char *fmt);
TINE_EXPORT short GetArrayTypeFromString(char *fmt);
TINE_EXPORT short GetArrayRowLengthFromString(char *fmt);
TINE_EXPORT char *GetArrayTypeAsString(int type);
TINE_EXPORT int isActiveTransferMode(short mode);
extern char *modetoa[];

#undef CF_TEXT
#undef CF_NULL
/** format specifications as tine defined constants */
#define TDC_CF_DOUBLE       512
#define TDC_CF_INT16        513
#define TDC_CF_SHORT        513
#define TDC_CF_INTEGER      513
#define TDC_CF_INT8         514
#define TDC_CF_BYTE         514
#define TDC_CF_INT32        515
#define TDC_CF_LONG         515
#define TDC_CF_TEXT         516
#define TDC_CF_CHAR         516
#define TDC_CF_FLOAT        517
#define TDC_CF_INT64        518
#define TDC_CF_DLONG        518
#define TDC_CF_STRUCT       519
#define TDC_CF_CHAR8        520
#define TDC_CF_NAME8        520
#define TDC_CF_STRING8      520
#define TDC_CF_CHAR16       521
#define TDC_CF_NAME16       521
#define TDC_CF_STRING16     521
#define TDC_CF_CHAR16FI     522
#define TDC_CF_NAME16FI     522
#define TDC_CF_BIT          523
#define TDC_CF_HITTS        524
#define TDC_CF_CHAR32       525
#define TDC_CF_NAME32       525
#define TDC_CF_STRING32     525
#define TDC_CF_FI           526
#define TDC_CF_FLTINT       526
#define TDC_CF_II           527
#define TDC_CF_LI           527
#define TDC_CF_LL           527
#define TDC_CF_INTINT       527
#define TDC_CF_LNGINT       527
#define TDC_CF_DI           528
#define TDC_CF_DBLINT       528
#define TDC_CF_SI           529
#define TDC_CF_SHTINT       529
#define TDC_CF_TDS          530
#define TDC_CF_IFI          530
#define TDC_CF_INTFLTINT    530
#define TDC_CF_CHAR48       531
#define TDC_CF_NAME48       531
#define TDC_CF_STRING48     531
#define TDC_CF_CHAR8I       532
#define TDC_CF_NAME8I       532
#define TDC_CF_CHAR16I      533
#define TDC_CF_NAME16I      533
#define TDC_CF_CHAR32I      534
#define TDC_CF_NAME32I      534
#define TDC_CF_CHAR48I      535
#define TDC_CF_NAME48I      535

#define TDC_CF_USTRING      536
#define TDC_CF_XYS          537
#define TDC_CF_FFI          537
#define TDC_CF_FLTFLTINT    537
#define TDC_CF_FII          538
#define TDC_CF_FLTINTINT    538
#define TDC_CF_FIS          538
#define TDC_CF_FILTER       539
#define TDC_CF_IFFF         539
#define TDC_CF_INTFLTFLTFLT 539
#define TDC_CF_SPECTRUM     540
#define TDC_CF_ADDRESS      541
#define TDC_CF_IIII         541
#define TDC_CF_INTINTINTINT 541
#define TDC_CF_WINDOW       542
#define TDC_CF_TTII         542
#define TDC_CF_POINT        543
#define TDC_CF_FF           543
#define TDC_CF_XY           543
#define TDC_CF_FLTFLT       543
#define TDC_CF_TDSI         544
#define TDC_CF_INTFLTSHTSHT 544
#define TDC_CF_BOOLEAN      545
#define TDC_CF_IIFF         546
#define TDC_CF_INTINTFLTFLT 546
#define TDC_CF_FWINDOW      546
#define TDC_CF_FIFI         547
#define TDC_CF_FLTINTFLTINT 547
#define TDC_CF_CHAR64       548
#define TDC_CF_NAME64       548
#define TDC_CF_STRING64     548
#define TDC_CF_CHAR64I      549
#define TDC_CF_NAME64I      549
#define TDC_CF_INTDBL       550
#define TDC_CF_LNGDBL       550
#define TDC_CF_FLTDBL       551
#define TDC_CF_DBLDBL       552
#define TDC_CF_NAMEDBLDBL   553
#define TDC_CF_NAME16DBLDBL 553
#define TDC_CF_NAME16INTINT 554
#define TDC_CF_CHAR16INTINT 554
#define TDC_CF_NAME16II     554
#define TDC_CF_CHAR16II     554
#define TDC_CF_STRING16II   554
#define TDC_CF_NAMEII       554
#define TDC_CF_INTINTINT    555
#define TDC_CF_III          555
#define TDC_CF_NAME32DBLDBL 556
#define TDC_CF_CHAR32DBLDBL 556
#define TDC_CF_STRING32DBLDBL 556
#define TDC_CF_NAME64DBLDBL 557
#define TDC_CF_CHAR64DBLDBL 557
#define TDC_CF_STRING64DBLDBL 557
#define TDC_CF_IFFFNAME     558
#define TDC_CF_IFFFNAME16   558
#define TDC_CF_INTFLTFLTFLTNAME16 558
#define TDC_CF_UNAME        558
#define TDC_CF_XML          559
#define TDC_CF_DBLDBLDBL    560
#define TDC_CF_NAME64DBLDBLDBL 561
#define TDC_CF_CHAR64DBLDBLDBL 561
#define TDC_CF_STRING64DBLDBLDBL 561
#define TDC_CF_BITFIELD8    562
#define TDC_CF_BITFIELD16   563
#define TDC_CF_BITFIELD32   564
#define TDC_CF_BITFIELD64   565
#define TDC_CF_NAME64DBL    566
#define TDC_CF_CHAR64DBL    566
#define TDC_CF_STRING64DBL  566
#define TDC_CF_IMAGE        567
#define TDC_CF_HISTORY      568
#define TDC_CF_STRING       569
#define TDC_CF_ASPECTRUM    570
#define TDC_CF_AIMAGE       571
#define TDC_CF_KEYVALUE     572
#define TDC_CF_DBLTIME      573
#define TDC_CF_NAME64TIME   574
#define TDC_CF_CHAR64TIME   574
#define TDC_CF_STRING64TIME 574
#define TDC_CF_MDA          575
#define TDC_CF_UINT16       576
#define TDC_CF_UINT32       577
#define TDC_CF_UINT64       578
#define TDC_CF_GSPECTRUM    579

#define TDC_CF_DEFAULT      766
#define TDC_CF_NULL         767

/** format specifications as enum */
enum TFormat
{
  CF_DOUBLE = 512,
  CF_INT16 = 513,
  CF_SHORT = 513,
  CF_INTEGER = 513,
  CF_INT8 = 514,
  CF_UINT8 = 514,
  CF_BYTE = 514,
  CF_INT32 = 515,
  CF_LONG = 515,
  CF_TEXT = 516,
  CF_CHAR = 516,
  CF_FLOAT = 517,
  CF_INT64 = 518,
  CF_DLONG = 518,
  CF_STRUCT = 519,
  CF_CHAR8 = 520,
  CF_NAME8 = 520,
  CF_STRING8 = 520,
  CF_CHAR16 = 521,
  CF_NAME16 = 521,
  CF_STRING16 = 521,
  CF_CHAR16FI = 522,
  CF_NAME16FI = 522,
  CF_BIT = 523,
  CF_HITTS = 524,
  CF_CHAR32 = 525,
  CF_NAME32 = 525,
  CF_STRING32 = 525,
  CF_FI = 526,
  CF_FLTINT = 526,
  CF_II = 527,
  CF_LI = 527,
  CF_LL = 527,
  CF_INTINT = 527,
  CF_LNGINT = 527,
  CF_DI = 528,
  CF_DBLINT = 528,
  CF_SI = 529,
  CF_SHTINT = 529,
  CF_TDS = 530,
  CF_IFI = 530,
  CF_INTFLTINT = 530,
  CF_CHAR48 = 531,
  CF_NAME48 = 531,
  CF_STRING48 = 531,
  CF_CHAR8I = 532,
  CF_NAME8I = 532,
  CF_CHAR16I = 533,
  CF_NAME16I = 533,
  CF_CHAR32I = 534,
  CF_NAME32I = 534,
  CF_CHAR48I = 535,
  CF_NAME48I = 535,
  CF_USTRING = 536,
  CF_XYS = 537,
  CF_FFI = 537,
  CF_FLTFLTINT = 537,
  CF_FII = 538,
  CF_FLTINTINT = 538,
  CF_FIS = 538,
  CF_FILTER = 539,
  CF_IFFF = 539,
  CF_INTFLTFLTFLT = 539,
  CF_SPECTRUM = 540,
  CF_ADDRESS = 541,
  CF_IIII = 541,
  CF_INTINTINTINT = 541,
  CF_WINDOW = 542,
  CF_TTII = 542,
  CF_POINT = 543,
  CF_FF = 543,
  CF_XY = 543,
  CF_FLTFLT = 543,
  CF_TDSI = 544,
  CF_INTFLTSHTSHT = 544,
  CF_BOOLEAN = 545,
  CF_IIFF = 546,
  CF_INTINTFLTFLT = 546,
  CF_FWINDOW = 546,
  CF_FIFI = 547,
  CF_FLTINTFLTINT = 547,
  CF_CHAR64 = 548,
  CF_NAME64 = 548,
  CF_STRING64 = 548,
  CF_CHAR64I = 549,
  CF_NAME64I = 549,
  CF_INTDBL = 550,
  CF_LNGDBL = 550,
  CF_FLTDBL = 551,
  CF_DBLDBL = 552,
  CF_NAMEDBLDBL = 553,
  CF_NAME16DBLDBL = 553,
  CF_NAME16INTINT = 554,
  CF_CHAR16INTINT = 554,
  CF_NAME16II = 554,
  CF_CHAR16II = 554,
  CF_STRING16II = 554,
  CF_NAMEII = 554,
  CF_INTINTINT = 555,
  CF_III = 555,
  CF_NAME32DBLDBL = 556,
  CF_CHAR32DBLDBL = 556,
  CF_STRING32DBLDBL = 556,
  CF_NAME64DBLDBL = 557,
  CF_CHAR64DBLDBL = 557,
  CF_STRING64DBLDBL = 557,
  CF_IFFFNAME = 558,
  CF_IFFFNAME16 = 558,
  CF_INTFLTFLTFLTNAME16 = 558,
  CF_UNAME = 558,
  CF_XML = 559,
  CF_DBLDBLDBL = 560,
  CF_NAME64DBLDBLDBL = 561,
  CF_CHAR64DBLDBLDBL = 561,
  CF_STRING64DBLDBLDBL = 561,
  CF_BITFIELD8 = 562,
  CF_BITFIELD16 = 563,
  CF_BITFIELD32 = 564,
  CF_BITFIELD64 = 565,
  CF_NAME64DBL = 566,
  CF_CHAR64DBL = 566,
  CF_STRING64DBL = 566,
  CF_IMAGE = 567,
  CF_HISTORY = 568,
  CF_STRING = 569,
  CF_ASPECTRUM = 570,
  CF_AIMAGE = 571,
  CF_KEYVALUE = 572,
  CF_DBLTIME = 573,
  CF_NAME64TIME = 574,
  CF_CHAR64TIME = 574,
  CF_STRING64TIME = 574,
  CF_MDA = 575,
  CF_UINT16 = 576,
  CF_UINT32 = 577,
  CF_UINT64 = 578,
  CF_GSPECTRUM = 579,

  CF_DEFAULT = 766,
  CF_NULL = 767,
};

#define HISTORY_HDR_SIZE 16
#define MDA_HDR_SIZE 376
#define SPECTRUM_HDR_SIZE 96
#define SPECTRUM_STD_SIZE 4096
#define GSPECTRUM_HDR_SIZE 244
#define GSPECTRUM_STD_SIZE 1048576
#define IMAGE_HDR_SIZE 188
#define IMAGE_STD_FRAMESIZE 500000
#define STRING_STD_CAPACITY 128

#define LFMT(fmt) (((fmt)%256)+TDC_CF_DOUBLE)
#define BFMT(fmt) ((fmt)%256)

#define fmtsizeof(fmt) getFormatSize(((fmt)%256)+TDC_CF_DOUBLE)

#define isPrimitiveFormat(fmt) (fmt <= TDC_CF_INT64 ? TRUE : FALSE)
#define isBitfieldFormat(fmt) (((fmt) >= TDC_CF_BITFIELD8 && (fmt) <= TDC_CF_BITFIELD64) ? TRUE : FALSE)
int isIntegerFormat(short fmt);
TINE_EXPORT int IsIntegerFormat(int fmt);
int isNumberFormat(short fmt);
TINE_EXPORT int IsNumberFormat(int fmt);
int isSimpleFormat(short fmt);
TINE_EXPORT int IsSimpleFormat(int fmt);
int isSimpleStringFormat(short fmt);
TINE_EXPORT int IsSimpleStringFormat(int fmt);
int isStringFormat(short fmt);
TINE_EXPORT int IsStringFormat(int fmt);
TINE_EXPORT int IsTaggableFormat(short fmt);
int isTaggableFormat(short fmt);
int getCarriedFormatSize(char *tag);
TINE_EXPORT int IsVariableLengthFormat(short fmt);
int isVariableLengthFormat(short fmt);
int isAdjustableLengthFormat(short fmt);
int isTimeFormat(short fmt);
int isQueueableFormat(short fmt);
int getFieldInFormat(int fmt, char *dev, char *tag, int fieldIndex, int *foffset, int *ffmt);
TINE_EXPORT int GetFieldInFormat(int fmt, char *dev, char *tag, int fieldIndex, int *foffset, int *ffmt);

#define getSimpleFormatType(txt) (\
  !strcmp(txt,"INT8")     ? TDC_CF_BYTE :\
  !strcmp(txt,"UINT8")    ? TDC_CF_BYTE :\
  !strcmp(txt,"BYTE")     ? TDC_CF_BYTE :\
  !strcmp(txt,"SHORT")    ? TDC_CF_INT16 :\
  !strcmp(txt,"INT16")    ? TDC_CF_INT16 :\
  !strcmp(txt,"UINT16")   ? TDC_CF_UINT16 :\
  !strcmp(txt,"LONG")     ? TDC_CF_INT32 :\
  !strcmp(txt,"INT32")    ? TDC_CF_INT32 :\
  !strcmp(txt,"UINT32")   ? TDC_CF_UINT32 :\
  !strcmp(txt,"FLOAT")    ? TDC_CF_FLOAT :\
  !strcmp(txt,"SINGLE")   ? TDC_CF_FLOAT :\
  !strcmp(txt,"DOUBLE")   ? TDC_CF_DOUBLE :\
  !strcmp(txt,"DLONG")    ? TDC_CF_INT64 :\
  !strcmp(txt,"INT64")    ? TDC_CF_INT64 :\
  !strcmp(txt,"UINT64")   ? TDC_CF_UINT64 :\
  !strcmp(txt,"STRUCT")   ? TDC_CF_STRUCT :\
  !strcmp(txt,"TEXT")     ? TDC_CF_TEXT :\
  !strcmp(txt,"CHAR")     ? TDC_CF_TEXT :\
  !strcmp(txt,"STRING")   ? TDC_CF_STRING :\
  !strcmp(txt,"NAME8")    ? TDC_CF_CHAR8 :\
  !strcmp(txt,"CHAR8")    ? TDC_CF_CHAR8 :\
  !strcmp(txt,"NAME16")   ? TDC_CF_CHAR16 :\
  !strcmp(txt,"CHAR16")   ? TDC_CF_CHAR16 :\
  !strcmp(txt,"CHAR32")   ? TDC_CF_CHAR32 :\
  !strcmp(txt,"NAME32")   ? TDC_CF_CHAR32 :\
  !strcmp(txt,"CHAR48")   ? TDC_CF_CHAR48 :\
  !strcmp(txt,"NAME48")   ? TDC_CF_CHAR48 :\
  !strcmp(txt,"CHAR64")   ? TDC_CF_CHAR64 :\
  !strcmp(txt,"NAME64")   ? TDC_CF_CHAR64 :\
  !strcmp(txt,"BIT")      ? TDC_CF_BIT :\
  !strcmp(txt,"BOOLEAN")  ? TDC_CF_BOOLEAN  :\
  !strcmp(txt,"XML")      ? TDC_CF_XML :\
  !strcmp(txt,"KEYVALUE") ? TDC_CF_KEYVALUE :\
  TDC_CF_NULL )

#define getCompoundFormatType(txt) (\
  !strcmp(txt,"NAME16FLTINT") ? TDC_CF_CHAR16FI :\
  !strcmp(txt,"NAME16FI") ? TDC_CF_CHAR16FI :\
  !strcmp(txt,"CHAR16FI") ? TDC_CF_CHAR16FI :\
  !strcmp(txt,"NAME8I")   ? TDC_CF_CHAR8I :\
  !strcmp(txt,"CHAR8I")   ? TDC_CF_CHAR8I :\
  !strcmp(txt,"NAME16I")  ? TDC_CF_CHAR16I :\
  !strcmp(txt,"CHAR16I")  ? TDC_CF_CHAR16I :\
  !strcmp(txt,"NAME32I")  ? TDC_CF_CHAR32I :\
  !strcmp(txt,"CHAR32I")  ? TDC_CF_CHAR32I :\
  !strcmp(txt,"NAME48I")  ? TDC_CF_CHAR48I :\
  !strcmp(txt,"CHAR48I")  ? TDC_CF_CHAR48I :\
  !strcmp(txt,"NAME64I")  ? TDC_CF_CHAR64I :\
  !strcmp(txt,"CHAR64I")  ? TDC_CF_CHAR64I :\
  !strcmp(txt,"HITTS")    ? TDC_CF_HITTS :\
  !strcmp(txt,"FI")       ? TDC_CF_FI :\
  !strcmp(txt,"FLTINT")   ? TDC_CF_FI :\
  !strcmp(txt,"FLTDBL")   ? TDC_CF_FLTDBL :\
  !strcmp(txt,"II")       ? TDC_CF_II :\
  !strcmp(txt,"INTINT")   ? TDC_CF_II :\
  !strcmp(txt,"LI")       ? TDC_CF_II :\
  !strcmp(txt,"LNGINT")   ? TDC_CF_II :\
  !strcmp(txt,"LNGDBL")   ? TDC_CF_LNGDBL :\
  !strcmp(txt,"INTDBL")   ? TDC_CF_LNGDBL :\
  !strcmp(txt,"DBLDBL")   ? TDC_CF_DBLDBL :\
  !strcmp(txt,"DBLTIME")   ? TDC_CF_DBLTIME :\
  !strcmp(txt,"NAMEDBLDBL") ? TDC_CF_NAMEDBLDBL :\
  !strcmp(txt,"NAME16DBLDBL") ? TDC_CF_NAMEDBLDBL :\
  !strcmp(txt,"NAME32DBLDBL") ? TDC_CF_NAME32DBLDBL :\
  !strcmp(txt,"NAME64DBL") ? TDC_CF_NAME64DBL :\
  !strcmp(txt,"NAME64DBLDBL") ? TDC_CF_NAME64DBLDBL :\
  !strcmp(txt,"NAME64DBLDBLDBL") ? TDC_CF_NAME64DBLDBLDBL :\
  !strcmp(txt,"NAME64TIME") ? TDC_CF_NAME64TIME :\
  !strcmp(txt,"NAME16II") ? TDC_CF_NAME16II :\
  !strcmp(txt,"CHAR16II") ? TDC_CF_NAME16II :\
  !strcmp(txt,"NAME16INTINT") ? TDC_CF_NAME16II :\
  !strcmp(txt,"DBLDBLDBL")   ? TDC_CF_DBLDBLDBL :\
  !strcmp(txt,"TDS")      ? TDC_CF_TDS :\
  !strcmp(txt,"IFI")      ? TDC_CF_TDS :\
  !strcmp(txt,"INTFLTINT")? TDC_CF_TDS :\
  !strcmp(txt,"USTRING")  ? TDC_CF_USTRING :\
  !strcmp(txt,"XYS")      ? TDC_CF_XYS :\
  !strcmp(txt,"FFI")      ? TDC_CF_XYS :\
  !strcmp(txt,"FLTFLTINT")? TDC_CF_XYS :\
  !strcmp(txt,"FLTINTINT")? TDC_CF_FII :\
  !strcmp(txt,"FIS")      ? TDC_CF_FII :\
  !strcmp(txt,"FILTER")   ? TDC_CF_FILTER :\
  !strcmp(txt,"IFFF")     ? TDC_CF_IFFF :\
  !strcmp(txt,"INTFLTFLTFLT")? TDC_CF_IFFF :\
  !strcmp(txt,"SPECTRUM") ? TDC_CF_SPECTRUM :\
  !strcmp(txt,"ASPECTRUM") ? TDC_CF_ASPECTRUM :\
  !strcmp(txt,"GSPECTRUM") ? TDC_CF_GSPECTRUM :\
  !strcmp(txt,"ADDRESS")  ? TDC_CF_ADDRESS  :\
  !strcmp(txt,"INTINTINTINT")? TDC_CF_ADDRESS  :\
  !strcmp(txt,"IIII")     ? TDC_CF_ADDRESS  :\
  !strcmp(txt,"INTINTINT")? TDC_CF_III  :\
  !strcmp(txt,"III")      ? TDC_CF_III  :\
  !strcmp(txt,"WINDOW")   ? TDC_CF_WINDOW  :\
  !strcmp(txt,"TTII")     ? TDC_CF_WINDOW  :\
  !strcmp(txt,"POINT")    ? TDC_CF_POINT  :\
  !strcmp(txt,"XY")       ? TDC_CF_POINT  :\
  !strcmp(txt,"FLTFLT")   ? TDC_CF_POINT  :\
  !strcmp(txt,"TDSI")     ? TDC_CF_TDSI  :\
  !strcmp(txt,"INTFLTSHTSHT")? TDC_CF_TDSI  :\
  !strcmp(txt,"DEFAULT")  ? TDC_CF_DEFAULT :\
  !strcmp(txt,"IIFF")     ? TDC_CF_IIFF :\
  !strcmp(txt,"INTINTFLTFLT")? TDC_CF_IIFF :\
  !strcmp(txt,"FIFI")     ? TDC_CF_FIFI :\
  !strcmp(txt,"FLTINTFLTINT")? TDC_CF_FIFI :\
  !strcmp(txt,"FWINDOW")   ? TDC_CF_FWINDOW  :\
  !strcmp(txt,"IFFFNAME")   ? TDC_CF_IFFFNAME  :\
  !strcmp(txt,"INTFLTFLTFLTNAME") ? TDC_CF_IFFFNAME  :\
  !strcmp(txt,"UNAME") ? TDC_CF_IFFFNAME  :\
  !strcmp(txt,"BITFIELD8") ? TDC_CF_BITFIELD8  :\
  !strcmp(txt,"BITFIELD16") ? TDC_CF_BITFIELD16 :\
  !strcmp(txt,"BITFIELD32") ? TDC_CF_BITFIELD32 :\
  !strcmp(txt,"IMAGE") ? TDC_CF_IMAGE :\
  !strcmp(txt,"AIMAGE") ? TDC_CF_AIMAGE :\
  !strcmp(txt,"HISTORY") ? TDC_CF_HISTORY :\
  !strcmp(txt,"MDA") ? TDC_CF_MDA :\
  TDC_CF_NULL )

/** transport mode specifications as tine defined constants */
#define TDC_CM_NULL         0x00
#define TDC_CM_CANCEL       0x00
#define TDC_CM_SINGLE       0x01
#define TDC_CM_REFRESH      0x02
#define TDC_CM_DATACHANGE   0x02
#define TDC_CM_POLL         0x03
#define TDC_CM_TIMER        0x03
#define TDC_CM_REGISTER     0x04
#define TDC_CM_RECEIVE      0x05
#define TDC_CM_EVENT        0x06
#define TDC_CM_NODUPLICATES 0x10
#define TDC_CM_SEEKPATTERN  0x20
#define TDC_CM_MISSPATTERN  0x40
#define TDC_CM_STREAM       0x80
#define TDC_CM_SYNCGROUP    0x100
#define TDC_CM_BCAST        0x200
#define TDC_CM_NETWORK      0x200
#define TDC_CM_MCAST        0x200
#define TDC_CM_USE_ON_ERROR 0x400
#define TDC_CM_USEONERROR   0x400
#define TDC_CM_RETRY        0x800
#define TDC_CM_GROUPED      0x1000
#define TDC_CM_CONNECT      0x2000
#define TDC_CM_WAIT         0x4000
#define TDC_CM_SYNCNOTIFY   0x8000

/** transport mode specifications as enum */
enum TModes
{
  CM_NULL = 0x00,
  CM_CANCEL = 0x00,
  CM_SINGLE = 0x01,
  CM_REFRESH = 0x02,
  CM_DATACHANGE = 0x02,
  CM_POLL = 0x03,
  CM_TIMER = 0x03,
  CM_REGISTER = 0x04,
  CM_RECEIVE = 0x05,
  CM_EVENT = 0x06,
  CM_NODUPLICATES = 0x10,
  CM_SEEKPATTERN = 0x20,
  CM_MISSPATTERN = 0x40,
  CM_STREAM = 0x80,
  CM_SYNCGROUP = 0x100,
  CM_BCAST = 0x200,
  CM_NETWORK = 0x200, /* same code as CA_NETWORK below */
  CM_MCAST = 0x200,
  CM_USE_ON_ERROR = 0x400,
  CM_USEONERROR = 0x400,
  CM_RETRY = 0x800,
  CM_GROUPED = 0x1000,
  CM_CONNECT = 0x2000, /* same code as CA_CONNECT below */
  CM_WAIT = 0x4000,
  CM_SYNCNOTIFY = 0x8000,
};

#define BASEMODE(m)  ((m) & 0x07)
#define TARGETMODE(m)  ((m) & 0x0f7f)
#define CM_REMOVEVOLATILE (CM_NODUPLICATES|CM_SEEKPATTERN|CM_MISSPATTERN|CM_USEONERROR|CM_GROUPED|CM_SYNCGROUP|CM_WAIT)

#define getAccessMode(txt) (\
	strstr(txt,"READ")    ? TDC_CM_SINGLE :\
	strstr(txt,"WRITE")   ? TDC_CM_SINGLE :\
	strstr(txt,"SINGLE")  ? TDC_CM_SINGLE :\
	strstr(txt,"REFRESH") ? TDC_CM_DATACHANGE :\
	strstr(txt,"DATACHANGE") ? TDC_CM_DATACHANGE :\
	strstr(txt,"POLL")    ? TDC_CM_TIMER :\
	strstr(txt,"TIMER")   ? TDC_CM_TIMER :\
	strstr(txt,"REGISTER")? TDC_CM_REGISTER :\
	strstr(txt,"RECEIVE") ? TDC_CM_RECEIVE :\
	strstr(txt,"EVENT")   ? TDC_CM_EVENT :\
	CM_CANCEL )

/** contract access specifications as tine defined constants */
#define TDC_CA_NULL         0x00
#define TDC_CA_READ         0x01
#define TDC_CA_WRITE        0x02
#define TDC_CA_REPEAT       0x04
#define TDC_CA_SYNC         0x08
#define TDC_CA_ALARM        0x10
#define TDC_CA_HIST         0x20
#define TDC_CA_FIRST        0x40
#define TDC_CA_LAST         0x80
#define TDC_CA_RETRY        0x100
#define TDC_CA_FORCEOUTPUT  0x100 
#define TDC_CA_NETWORK      0x200
#define TDC_CA_MCAST        0x200
#define TDC_CA_QUERY        0x400
#define TDC_CA_CCSHOWALL    0x400 
#define TDC_CA_LOCKED       0x800
#define TDC_CA_XREAD        0x800
#define TDC_CA_STREAM       0x800
#define TDC_CA_NORETRY      0x1000
#define TDC_CA_NOSYNC       0x1000 
#define TDC_CA_CONNECT      0x2000
#define TDC_CA_MUTABLE      0x4000
#define TDC_CA_FIXTIMESTAMP 0x8000 

/** contract access specifications as enum
 *  first byte of assigned access in carried in the call
 */
enum TAccess
{
  CA_NULL = 0x00,           /**< unassigned */
  CA_READ = 0x01,           /**< READ access (cln and srv) */
  CA_WRITE = 0x02,          /**< WRITE access (cln and srv) */
  CA_REPEAT = 0x04,         /**< call is being repeated (srv: passed to eqm) */
  CA_SYNC = 0x08,           /**< call is marked synchronous (cln: guarantee sync) */
  CA_ALARM = 0x10,          /**< call is from local alarm system (srv: passed to eqm) */
  CA_HIST = 0x20,           /**< call is from local history system (srv: passed to eqm) */
  CA_FIRST = 0x40,          /**< link is coming into scope (srv: passed to eqm) */
  CA_MT = 0x40,             /**< property registration flags runInSeparateThread = TRUE (from prp query fcn) */
  CA_LAST = 0x80,           /**< link is going out of scope (srv: passed to eqm) */
  CA_STATIC = 0x80,         /**< property registration contains static information */
  CA_NOCALLBACKS = 0x80,    /**< signal to suspend callbacks during a synchronous call (cln: passed to link call) */
  CA_RETRY = 0x100,         /**< call should retry on timeout (cln: passed to link call) */
  CA_FORCEOUTPUT = 0x100,   /**< property registration enforces data length and format */
  CA_NETWORK = 0x200,       /**< property registration enforces NETWORK access */
  CA_MCAST = 0x200,         /**< synonym to CA_NETWORK (cln: passed to link call) */
  CA_QUERY = 0x400,         /**< call is an internal query (srv: passed to eqm) */
  CA_SINGLE = 0x400,        /**< signal that access is single synchronous (contract interpretation) */
  CA_CCSHOWALL = 0x400,     /**< sync call should pass all relevant completion code flags (cln: passed to link call) */
  CA_LOCKED = 0x800,        /**< call has an access lock (srv: passed to eqm) */
  CA_STREAM = 0x800,        /**< sync call should use CM_STREAM access (cln: passed to link call) */
  CA_XREAD = 0x800,         /**< property registration requests exclusive READ access */
  CA_NORETRY = 0x1000,      /**< call should NOT retry on timeout (cln: passed to link call) */
  CA_NOSYNC = 0x1000,       /**< property registration enforces asynchronous access */
  CA_CONNECT = 0x2000,      /**< sync call should use CM_CONNECT access (cln: passed to link call) */
  CA_MUTABLE = 0x4000,      /**< sync call can change the input parameters (cln: passed to link call) */
  CA_SAVERESTORE = 0x4000,  /**< property registration signals save-and-restore property */
  CA_FIXTIMESTAMP = 0x8000, /**< incoming timestamp should be fixed to local time (cln: passed to link call) */
  CA_INDEPENDENT = 0x8000,  /**< property registration signals WRITE access independent of equipment module data */
  CA_SENDCODE = 0x8000,     /**< special signal for scheduling (srv: passed to SystemSchedulePropertyEx) */
  CA_FIX = 0xff,            /**< special signal for property query functions */
};
#define getAccessType(txt) ( strstr(txt,"WRITE") ? TDC_CA_WRITE : TDC_CA_READ )

/**< graph types as tine defined constants */
#define TDC_GT_LIN  0
#define TDC_GT_LOG  1
#define TDC_GT_SLOG 2
#define TDC_GT_TD   3
#define TDC_GT_BIT  4
#define TDC_GT_TEXT 5
#define TDC_GT_ABS  6
#define TDC_GT_HEX  7

/**< graph types as enum */
enum TGraphType
{
  GT_LIN = 0,
  GT_LOG = 1,
  GT_SLOG = 2,
  GT_TD = 3,
  GT_BIT = 4,
  GT_TEXT = 5,
  GT_ABS = 6,
  GT_HEX = 7,
};
#define getGraphType(txt) (\
  txt == NULL           ? TDC_GT_LIN:\
  strstr(txt,"LIN")     ? TDC_GT_LIN :\
  strstr(txt,"LOG")     ? TDC_GT_LOG :\
  strstr(txt,"SLOG")    ? TDC_GT_SLOG :\
  strstr(txt,"TD")      ? TDC_GT_TD :\
  strstr(txt,"BIT")     ? TDC_GT_BIT :\
  strstr(txt,"TEXT")    ? TDC_GT_TEXT :\
  strstr(txt,"ABS")     ? TDC_GT_ABS :\
  strstr(txt,"HEX")     ? TDC_GT_HEX :\
  TDC_GT_LIN )

/** array type specifications as enum */
enum TArrayType
{
  AT_UNKNOWN = 0x00,
  AT_SCALAR = 0x01,
  AT_SINGLE = 0x02,
  AT_DOUBLE = 0x04,
  AT_BOOLEAN = 0x08,
  AT_CHANNEL = 0x10,
  AT_SPECTRUM = 0x20,
  AT_TRACE = 0x20,
  AT_WAVEFORM = 0x20,
  AT_STRUCTURED = 0x40,
  AT_COLLECTION = 0x40,
  AT_UNSTRUCTURED = 0x80,
  AT_BLOB = 0x80,
  AT_IMAGE = 0x80,
  AT_JPEG = 0x100,
  AT_MPEG = 0x200,
  AT_BMP = 0x400,
  AT_GIF = 0x800,
  AT_WAV = 0x1000,
  AT_MP3 = 0x2000,
};

/* image format definitions */
enum TImageFormat
{
  IF_GRAY = 0,
  IF_MIN = 0,
  IF_RGB = 1,
  IF_RGBA = 2,
  IF_RGB15 = 3,
  IF_RGB16 = 4,
  IF_JPEG = 5,
  IF_TIFF = 6,
  IF_YUV411 = 7,
  IF_YUV422 = 8,
  IF_YUV444 = 9,
  IF_HUFFYUV = 10,
  IF_BITMAP = 11,  /**< Windows BitMaP bits follow headers */
  IF_BAYER = 12,    /**< Bayer-packed image bits */
  IF_PNG = 13,
  IF_BASE64 = 14,
  IF_MAX = 14,
};

#define CF_IMAGE_BASETAG 0x33565356 

/** image flags */
enum TImageFlags
{
  IFLG_LITTLE_ENDIAN = 0x1,
  IFLG_LOSSLESS_COMP = 0x2,
  IFLG_HORIZONTAL_FLIP = 0x4,
  IFLG_VERTICAL_FLIP = 0x8,
};

/* property signals as tine defined constants */
#define TDC_PS_NONE    0x00 /**< NULL signal */
#define TDC_PS_ACCESS  0x01 /**< property is being accessed by a caller */
#define TDC_PS_RETRY   0x02 /**< property is being retried */
#define TDC_PS_LATE    0x04 /**< property is being returned late */
#define TDC_PS_PENDING 0x08 /**< property is being called while last transmission still pending */
#define TDC_PS_SENT    0x10 /**< property has been sent to caller */
#define TDC_PS_ALL     0xff /**< signal on any event */

#ifdef NEED_RELEASE4_DEFINES
#define PS_NONE    0x00 /**< NULL signal */
#define PS_ACCESS  0x01 /**< property is being accessed by a caller */
#define PS_RETRY   0x02 /**< property is being retried */
#define PS_LATE    0x04 /**< property is being returned late */
#define PS_PENDING 0x08 /**< property is being called while last transmission still pending */
#define PS_SENT    0x10 /**< property has been sent to caller */
#define PS_ALL     0xff /**< signal on any event */
#endif

/** property signals as enum */
enum TPropertySignals
{
  CS_NONE = 0x00,      /**< NULL signal */
  CS_ACCESS = 0x01,    /**< property is being accessed by a caller */
  CS_RETRY = 0x02,     /**< property is being retried */
  CS_LATE = 0x04,      /**< property is being returned late */
  CS_PENDING = 0x08,   /**< property is being called while last transmission still pending */
  CS_SENT = 0x10,      /**< property has been sent to caller */
  CS_CALLED = 0x20,    /**< property has been called and result returned */
  CS_PROCESSED = 0x40, /**< property's returned data has been processed */
  CS_ALL = 0xff,       /**< signal on any event */
};

#define RB_USE_ERRORVALUE 0x01 /**< use supplied value as data set on error */
#define RB_USE_TIMESTAMP  0x02 /**< use local timestamp instead of incoming */

#define CSHOSTS_FILE  "cshosts.csv"
#define FECID_FILE    "fecid.csv"
#define EXPORT_FILE   "exports.csv"
#define EQPDB_FILE    "eqpdbase.csv"
#define FECDB_FILE    "fecaddr.csv"
#define USERS_FILE    "users.csv"
#define DENYUSER_FILE "denyuser.csv"
#define IPNETS_FILE   "ipnets.csv"
#define DENYNETS_FILE "denyip.csv"
#define IPBCASTNETS_FILE "ipbcast.csv"
#define IPXNETS_FILE  "ipxnets.csv"
#define IPXBCASTNETS_FILE "ipxbcast.csv"
#define ALIAS_FILE    "alias.csv"
#define ADMINS_FILE   "admins.csv"
#define REGIONRULES_FILE "regionrules.csv"
#define SELFTEST_FILE "selftest"

#ifndef FOLLOWERS_FILE
# define FOLLOWERS_FILE "followers.csv"
#endif
#define FOLLOWERS_BUFFER_SPACE 25

#define FECNAME_UNKNOWN "*unknown*"

#define MAJOR_VERSION 5
#define MINOR_VERSION 3
#define VERSION_REVISION 0

extern int tineBuildId;

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* _TINEDEFS_H_ */
